uniform sampler2D colorTexture;
uniform sampler2D height_map;
//uniform sampler2D shadow_map; // old shadow map
uniform sampler2DShadow shadow_map;
uniform sampler2D normal_map;
uniform float alpha;
uniform int useSMfrag; //use shadow mapping
uniform int useNMfrag; //use normal mapping
uniform int usePMfrag; //use parallax normal mapping
uniform int NoEnlighting;  // use texture color only (for sky)
varying vec4 SM_tex_coord; // passed on from VS
varying vec2 texel;
varying vec3 normal;
varying vec3 lightVec;
varying vec3 halfVec;
varying vec3 eyeVec;

varying vec3 eyeVec4prlx; // for parallax

void main(void)
{
	vec2 frTexel=texel;

	if(NoEnlighting==1)
	{
		gl_FragColor = texture2D( colorTexture, frTexel);
		gl_FragColor = vec4(gl_FragColor.xyz,alpha);
		return;
	}

	vec2 prlxTexel; //parallax
	if(usePMfrag==1) //parallax
	{
		vec4 ht = texture2D( height_map, frTexel);				//parallax
		float scale=0.02;
		ht.x=ht.x*scale -0.5*scale;
		prlxTexel=frTexel.xy+ht.x*(-eyeVec4prlx.xy/eyeVec.z);		// /eyeVec.z	//parallax
		gl_FragColor = texture2D( colorTexture, prlxTexel);		//parallax
		frTexel = prlxTexel;
	}
	
	if (useSMfrag==1)
	{
		//SM_tex_coord.z=SM_tex_coord.z-0.004*SM_tex_coord.w;
		float shadow = textureProj(shadow_map, SM_tex_coord);
		//float shadow = shadow2DProj(shadow_map, SM_tex_coord).x;
		
		if (shadow<=0.1)
			gl_FragColor=0.5*texture2D( colorTexture, frTexel);
		else
			gl_FragColor = texture2D( colorTexture, frTexel);
		
		// gl_FragColor = shadow*texture2D( colorTexture, frTexel); // almost black color
	}
	else
		gl_FragColor = texture2D( colorTexture, frTexel);

	// initial value of gl_FragColor should be set by now

	gl_FragColor = vec4(gl_FragColor.xyz,alpha);
	//gl_FragColor = texture2D( colorTexture, frTexel);
	//return; // for debug (disable lighting)

	vec3 N;
	if (useNMfrag==1)
	{
		vec4 directT;
		// lookup normal from normal map, move from [0,1] to  [-1, 1] range, normalize
		directT = texture2D (normal_map, frTexel);
		
		vec3 nn = 2.0 * directT.rgb - 1.0;
		N = normalize (nn);
	}
	else
		N = normal; // normal does not need to be displaced if parallax mapping used without normal mapping, cause all pixels will have same normal when interpolated

	// value of normal vector should be set by now
	
	/****/
	// lighting starts here. values are either in eye space or tangent space, depending on whether normal mapping is enabled or not	

	vec4 sceneColor = gl_FragColor;	
	vec4 light_ambient=vec4(0.1,0.1,0.1,1);
	//vec4 light_diffuse=vec4(1,1,1,1);
	//vec4 light_specular=vec4(1,1,1,1);
	vec4 light_diffuse=vec4(0.8,0.8,0.8,1);
	vec4 light_specular=vec4(0.5,0.5,0.5,1);
	vec4 matr_ambient=sceneColor;
	vec4 matr_diffuse=sceneColor;
	vec4 matr_specular=vec4(0.6,0.6,0.6,1.0);
	float shininess = 30.0;

	vec4 final_color =(sceneColor * matr_ambient) + (light_ambient * matr_ambient);
		
	float lambertTerm = max( dot(lightVec, N), 0.0);
	if(lambertTerm > 0.0)
	{
		final_color += light_diffuse * matr_diffuse *lambertTerm;
		
		float specular = pow( max(dot(halfVec, normal), 0.0),shininess );

		final_color += light_specular *matr_specular * specular;
	}

	gl_FragColor = final_color;

}